@@ -3,7 +3,7 @@
 *                         E v e n t   D i s p a t c h e r                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2006,2017 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2006,2018 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU Lesser General Public License as published by   *
@@ -116,7 +116,7 @@ public:
   virtual FXbool dispatchHandle(FXInputHandle hnd,FXuint mode);
 
   /// Dispatch when timeout expires
-  virtual FXbool dispatchTimeout(FXTime now);
+  virtual FXbool dispatchTimeout(FXTime due);
 
   /// Dispatch when idle
   virtual FXbool dispatchIdle();
