@@ -3,7 +3,7 @@
 *                       H a s h   T a b l e   C l a s s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2003,2017 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2003,2018 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU Lesser General Public License as published by   *
@@ -173,6 +173,12 @@ public:
   /// Copy constructor
   FXHashOf(const FXHashOf<KEYTYPE,VALUETYPE>& src):FXHash(src){}
 
+  /// Assignment operator
+  FXHashOf<KEYTYPE,VALUETYPE>& operator=(const FXHashOf<KEYTYPE,VALUETYPE>& other){ return reinterpret_cast<FXHashOf<KEYTYPE,VALUETYPE>&>(FXHash::operator=(other)); }
+
+  /// Adopt objects from orig, leaving orig empty
+  FXHashOf<KEYTYPE,VALUETYPE>& adopt(FXHashOf<KEYTYPE,VALUETYPE>& src){ return reinterpret_cast<FXHashOf<KEYTYPE,VALUETYPE>&>(FXHash::adopt(src)); }
+
   /// Return reference to slot assocated with given key
   VALUETYPE*& at(KEYTYPE* name){ return (VALUETYPE*&)FXHash::at((FXptr)name); }
 
