@@ -3,7 +3,7 @@
 *                        F i l e   S t a t i s t i c s                          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2005,2016 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2005,2018 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU Lesser General Public License as published by   *
@@ -21,7 +21,6 @@
 #ifndef FXSTAT_H
 #define FXSTAT_H
 
-
 namespace FX {
 
 
@@ -43,6 +42,9 @@ private:
   FXlong  fileSize;             /// File size
 public:
 
+  /// Initialize
+  FXStat():modeFlags(0),userNumber(0),groupNumber(0),linkCount(0),createTime(0),accessTime(0),modifyTime(0),fileVolume(0),fileIndex(0),fileSize(0){ }
+
   /// Return the mode flags for this file
   FXuint mode() const { return modeFlags; }
 
@@ -94,6 +96,9 @@ public:
   /// Return true if the file has the sticky bit set
   FXbool isSetSticky() const;
 
+  /// Return true if special device (character or block device)
+  FXbool isDevice() const;
+
   /// Return true if character device
   FXbool isCharacter() const;
 
