@@ -1,9 +1,9 @@
 /********************************************************************************
 *                                                                               *
-*                    M u l t i - L i ne   T e x t   W i d g e t                 *
+*                   M u l t i - L i n e   T e x t   W i d g e t                 *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2015 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2018 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU Lesser General Public License as published by   *
@@ -36,7 +36,8 @@ enum {
   TEXT_FIXEDWRAP     = 0x00800000,      /// Fixed wrap columns
   TEXT_NO_TABS       = 0x01000000,      /// Insert spaces for tabs
   TEXT_AUTOINDENT    = 0x02000000,      /// Autoindent
-  TEXT_SHOWACTIVE    = 0x04000000       /// Show active line
+  TEXT_SHOWACTIVE    = 0x04000000,      /// Show active line
+  TEXT_SHOWMATCH     = 0x08000000,      /// Show matching brace
   };
 
 
@@ -68,6 +69,19 @@ struct FXTextChange {
   };
 
 
+/**
+* Text selection data.
+* A range-selection is in effect when startpos<endpos, while a block-select
+* is in effect when startcol<endcol additionally.
+*/
+struct FXTextSelection {
+  FXint  startpos;      /// Start of selection (begin of first line if block-select)
+  FXint  endpos;        /// End of selection (start of line past last if block-select)
+  FXint  startcol;      /// Start column, if block-select (actually, indent)
+  FXint  endcol;        /// End column, if block select
+  };
+
+
 /**
 * The text widget provides a multi-line text editing control.
 * The widget supports both clipboard for cut-and-paste operations,
@@ -131,113 +145,127 @@ struct FXTextChange {
 * The flag TEXT_NO_TABS causes a TAB key to be replaced by the appropriate
 * number of spaces when entered interactively; however, the user can still
 * enter a TAB by entering Ctrl+TAB.
-* Finally, the TEXT_SHOWACTIVE mode will cause the line containing the current
+* The TEXT_SHOWACTIVE mode will cause the line containing the current
 * cursor location to be highlighted.  This makes it easier to find the insertion
 * point when scrolling back and forth a large document.
+* The flag TEXT_SHOWMATCH will briefly flash the matching brace, parenthesis, or
+* bracket when the cursor is positioned at brace, parenthesis, or bracket.
 */
 class FXAPI FXText : public FXScrollArea {
   FXDECLARE(FXText)
 protected:
-  FXchar        *buffer;              // Text buffer being edited
-  FXchar        *sbuffer;             // Text style buffer
-  FXint         *visrows;             // Starts of rows in buffer
-  FXint          length;              // Length of the actual text in the buffer
-  FXint          nvisrows;            // Number of visible rows
-  FXint          nrows;               // Total number of rows
-  FXint          gapstart;            // Start of the insertion point (the gap)
-  FXint          gapend;              // End of the insertion point+1
-  FXint          toppos;              // Start position of first visible row
-  FXint          toprow;              // Row number of first visible row
-  FXint          keeppos;             // Position to keep on top visible row
-  FXint          selstartpos;         // Start of selection
-  FXint          selendpos;           // End of selection
-  FXint          hilitestartpos;      // Hightlight start position
-  FXint          hiliteendpos;        // Hightlight end position
-  FXint          anchorpos;           // Anchor position
-  FXint          cursorpos;           // Cursor position
-  FXint          cursorstartpos;      // Cursor row start pos
-  FXint          cursorendpos;        // Cursor row end pos
-  FXint          cursorrow;           // Cursor row
-  FXint          cursorcol;           // Cursor column indent (not character offset!)
-  FXint          prefcol;             // Preferred cursor column
-  FXint          margintop;           // Margins top
-  FXint          marginbottom;        // Margin bottom
-  FXint          marginleft;          // Margin left
-  FXint          marginright;         // Margin right
-  FXint          wrapwidth;           // Wrap width in pixels
-  FXint          wrapcolumns;         // Wrap columns
-  FXint          tabwidth;            // Tab width in pixels
-  FXint          tabcolumns;          // Tab columns
-  FXint          barwidth;            // Line number width
-  FXint          barcolumns;          // Line number columns
-  FXFont        *font;                // Text font
-  FXColor        textColor;           // Normal text color
-  FXColor        selbackColor;        // Select background color
-  FXColor        seltextColor;        // Select text color
-  FXColor        hilitebackColor;     // Highlight background color
-  FXColor        hilitetextColor;     // Highlight text color
-  FXColor        activebackColor;     // Background color for active line
-  FXColor        numberColor;         // Line number color
-  FXColor        cursorColor;         // Cursor color
-  FXColor        barColor;            // Bar background color
-  FXint          textWidth;           // Total width of all text
-  FXint          textHeight;          // Total height of all text
-  const FXchar  *delimiters;          // Delimiters
-  FXString       clipped;             // Clipped text
-  FXint          vrows;               // Default visible rows
-  FXint          vcols;               // Default visible columns
-  FXString       help;                // Status line help
-  FXString       tip;                 // Tooltip
-  FXHiliteStyle *hilitestyles;        // Style definitions
-  FXTime         matchtime;           // Match time (ns)
-  FXint          grabx;               // Grab point x
-  FXint          graby;               // Grab point y
-  FXuchar        mode;                // Mode widget is in
-  FXbool         modified;            // User has modified text
+  FXchar         *buffer;               // Text buffer being edited
+  FXchar         *sbuffer;              // Text style buffer
+  FXint          *visrows;              // Starts of rows in buffer
+  FXint           length;               // Length of the actual text in the buffer
+  FXint           nvisrows;             // Number of visible rows
+  FXint           nrows;                // Total number of rows
+  FXint           gapstart;             // Start of the insertion point (the gap)
+  FXint           gapend;               // End of the insertion point+1
+  FXint           toppos;               // Start position of first visible row
+  FXint           toprow;               // Row number of first visible row
+  FXint           keeppos;              // Position to keep on top visible row
+  FXTextSelection select;               // Text selection
+  FXTextSelection hilite;               // Text highlight
+  FXint           anchorpos;            // Anchor position
+  FXint           anchorrow;            // Anchor row
+  FXint           anchorcol;            // Anchor column (kept inside text)
+  FXint           anchorvcol;           // Unconstrained anchor column
+  FXint           cursorpos;            // Cursor position
+  FXint           cursorrow;            // Cursor row
+  FXint           cursorcol;            // Cursor column (kept inside text)
+  FXint           cursorvcol;           // Unconstrained cursor column
+  FXint           prefcol;              // Preferred cursor column
+  FXint           margintop;            // Margins top
+  FXint           marginbottom;         // Margin bottom
+  FXint           marginleft;           // Margin left
+  FXint           marginright;          // Margin right
+  FXint           wrapwidth;            // Wrap width in pixels
+  FXint           wrapcolumns;          // Wrap columns
+  FXint           tabwidth;             // Tab width in pixels
+  FXint           tabcolumns;           // Tab columns
+  FXint           barwidth;             // Line number width
+  FXint           barcolumns;           // Line number columns
+  FXFont         *font;                 // Text font
+  FXColor         textColor;            // Normal text color
+  FXColor         selbackColor;         // Select background color
+  FXColor         seltextColor;         // Select text color
+  FXColor         hilitebackColor;      // Highlight background color
+  FXColor         hilitetextColor;      // Highlight text color
+  FXColor         activebackColor;      // Background color for active line
+  FXColor         numberColor;          // Line number color
+  FXColor         cursorColor;          // Cursor color
+  FXColor         barColor;             // Bar background color
+  FXint           textWidth;            // Total width of all text
+  FXint           textHeight;           // Total height of all text
+  const FXchar   *delimiters;           // Delimiters
+  FXString        clipped;              // Clipped text
+  FXint           vrows;                // Default visible rows
+  FXint           vcols;                // Default visible columns
+  FXString        help;                 // Status line help
+  FXString        tip;                  // Tooltip
+  FXHiliteStyle  *hilitestyles;         // Style definitions
+  FXuint          blink;                // Next cursor blink state
+  FXTime          matchtime;            // Match time (ns)
+  FXint           grabx;                // Grab point x
+  FXint           graby;                // Grab point y
+  FXuchar         mode;                 // Mode widget is in
+  FXbool          modified;             // User has modified text
 protected:
   FXText();
-  void calcVisRows(FXint s,FXint e);
-  void drawCursor(FXuint state);
-  virtual void paintCursor(FXDCWindow& dc) const;
-  virtual void eraseCursor(FXDCWindow& dc) const;
-  virtual void eraseCursorOverhang();
-  virtual FXuint styleOf(FXint beg,FXint end,FXint row,FXint col,FXint pos) const;
-  virtual void drawBufferText(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h,FXint pos,FXint n,FXuint style) const;
-  virtual void fillBufferRect(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h,FXuint style) const;
-  virtual void drawTextRow(FXDCWindow& dc,FXint row) const;
-  virtual void drawContents(FXDCWindow& dc) const;
-  virtual void drawNumbers(FXDCWindow& dc) const;
-  FXint posToLine(FXint pos,FXint ln) const;
-  void updateRange(FXint beg,FXint end) const;
   void movegap(FXint pos);
   void sizegap(FXint sz);
   void squeezegap();
   FXint charWidth(FXwchar ch,FXint indent) const;
+  FXint xoffset(FXint start,FXint pos) const;
   FXint wrap(FXint start) const;
+  FXint rowFromPos(FXint pos) const;
+  FXint posFromRow(FXint row) const;
+  FXint columnFromPos(FXint start,FXint pos) const;
+  FXint posFromColumn(FXint start,FXint col) const;
+  FXbool isdelimiter(FXwchar w) const;
   FXint measureText(FXint start,FXint end,FXint& wmax,FXint& hmax) const;
-  FXint lineWidth(FXint pos,FXint n) const;
-  FXint changeBeg(FXint pos) const;
-  FXint changeEnd(FXint pos) const;
-  FXint indentFromPos(FXint start,FXint pos) const;
-  FXint posFromIndent(FXint start,FXint indent) const;
-  void mutation(FXint pos,FXint ncins,FXint ncdel,FXint nrins,FXint nrdel);
-  virtual void replace(FXint pos,FXint m,const FXchar *text,FXint n,FXint style);
+  void calcVisRows(FXint s,FXint e);
   void recompute();
   FXint matchForward(FXint pos,FXint end,FXwchar l,FXwchar r,FXint level) const;
   FXint matchBackward(FXint pos,FXint beg,FXwchar l,FXwchar r,FXint level) const;
   FXint findMatching(FXint pos,FXint beg,FXint end,FXwchar ch,FXint level) const;
   void flashMatching();
   void moveContents(FXint x,FXint y);
+  FXint changeBeg(FXint pos) const;
+  FXint changeEnd(FXint pos) const;
+  void mutation(FXint pos,FXint ncins,FXint ncdel,FXint nrins,FXint nrdel);
   FXint overstruck(FXint start,FXint end,const FXchar *text,FXint n);
-  void enterText(const FXchar *text,FXint n,FXbool notify);
-  void enterText(const FXString& text,FXbool notify);
+  void drawCursor(FXuint state);
+  virtual void paintCursor(FXDCWindow& dc) const;
+  virtual void eraseCursor(FXDCWindow& dc) const;
+  virtual void eraseCursorOverhang();
+  virtual void drawBufferText(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h,FXint pos,FXint n,FXuint style) const;
+  virtual void fillBufferRect(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h,FXuint style) const;
+  virtual FXuint styleOf(FXint beg,FXint end,FXint row,FXint col,FXint pos) const;
+  virtual void drawTextRow(FXDCWindow& dc,FXint row) const;
+  virtual void drawContents(FXDCWindow& dc) const;
+  virtual void drawNumbers(FXDCWindow& dc) const;
+  virtual void replace(FXint pos,FXint m,const FXchar *text,FXint n,FXint style);
+  void updateRows(FXint startrow,FXint endrow) const;
+  void updateRange(FXint startpos,FXint endpos) const;
   FXbool deletePendingSelection(FXbool notify);
+  FXint countTextLines(const FXchar* text,FXint n) const;
+  FXint countTextColumns(const FXchar* text,FXint n) const;
+  FXString tabbify(const FXchar* str,FXint len,FXint indent=0,FXint outdent=0,FXint shift=0,FXbool tab=false) const;
+  FXString tabbify(const FXString& str,FXint indent=0,FXint outdent=0,FXint shift=0,FXbool tab=false) const;
+  FXString extractTextBlock(FXint startpos,FXint endpos,FXint startcol,FXint endcol) const;
+  FXint removeTextBlock(FXint startpos,FXint endpos,FXint startcol,FXint endcol,FXbool notify);
+  void replaceTextBlock(FXint startpos,FXint endpos,FXint startcol,FXint endcol,const FXString& text,FXbool notify);
+  FXint shiftText(FXint startpos,FXint endpos,FXint shift,FXbool notify);
+  FXint caseShift(FXint startpos,FXint endpos,FXint upper,FXbool notify);
 protected:
   enum {
     MOUSE_NONE,                 // No mouse operation
     MOUSE_CHARS,                // Selecting characters
     MOUSE_WORDS,                // Selecting words
     MOUSE_LINES,                // Selecting lines
+    MOUSE_BLOCK,                // Select block
     MOUSE_SCROLL,               // Scrolling
     MOUSE_DRAG,                 // Dragging text
     MOUSE_TRYDRAG               // Tentative drag
@@ -262,6 +290,8 @@ private:
   FXText& operator=(const FXText&);
 public:
   long onPaint(FXObject*,FXSelector,void*);
+  long onEnter(FXObject*,FXSelector,void*);
+  long onLeave(FXObject*,FXSelector,void*);
   long onBlink(FXObject*,FXSelector,void*);
   long onFlash(FXObject*,FXSelector,void*);
   long onFocusIn(FXObject*,FXSelector,void*);
@@ -301,6 +331,7 @@ public:
   long onUpdHaveSelection(FXObject*,FXSelector,void*);
   long onUpdHaveEditableSelection(FXObject*,FXSelector,void*);
   long onIMEStart(FXObject*,FXSelector,void*);
+  long onTipTimer(FXObject*,FXSelector,void*);
 
   // Value access
   long onCmdSetStringValue(FXObject*,FXSelector,void*);
@@ -319,8 +350,6 @@ public:
   long onCmdCursorPageDown(FXObject*,FXSelector,void*);
   long onCmdCursorWordLeft(FXObject*,FXSelector,void*);
   long onCmdCursorWordRight(FXObject*,FXSelector,void*);
-  long onCmdCursorSelStart(FXObject*,FXSelector,void*);
-  long onCmdCursorSelEnd(FXObject*,FXSelector,void*);
 
   // Cursor drag movement
   long onCmdCursorShiftTop(FXObject*,FXSelector,void*);
@@ -356,9 +385,9 @@ public:
   long onCmdCutSel(FXObject*,FXSelector,void*);
   long onCmdCopySel(FXObject*,FXSelector,void*);
   long onCmdPasteSel(FXObject*,FXSelector,void*);
+  long onCmdPasteMiddle(FXObject*,FXSelector,void*);
   long onCmdDeleteSel(FXObject*,FXSelector,void*);
   long onCmdReplaceSel(FXObject*,FXSelector,void*);
-  long onCmdPasteMiddle(FXObject*,FXSelector,void*);
   long onCmdSelectChar(FXObject*,FXSelector,void*);
   long onCmdSelectWord(FXObject*,FXSelector,void*);
   long onCmdSelectLine(FXObject*,FXSelector,void*);
@@ -380,6 +409,10 @@ public:
   // Control commands
   long onCmdShiftText(FXObject*,FXSelector,void*);
   long onCmdChangeCase(FXObject*,FXSelector,void*);
+  long onCmdCopyLine(FXObject*,FXSelector,void*);
+  long onCmdMoveLineUp(FXObject*,FXSelector,void*);
+  long onCmdMoveLineDown(FXObject*,FXSelector,void*);
+  long onCmdJoinLines(FXObject*,FXSelector,void*);
   long onCmdBlockBeg(FXObject*,FXSelector,void*);
   long onCmdBlockEnd(FXObject*,FXSelector,void*);
   long onCmdGotoMatching(FXObject*,FXSelector,void*);
@@ -420,8 +453,6 @@ public:
     ID_CURSOR_PAGEDOWN,
     ID_CURSOR_WORD_LEFT,
     ID_CURSOR_WORD_RIGHT,
-    ID_CURSOR_SEL_START,
-    ID_CURSOR_SEL_END,
     ID_CURSOR_SHIFT_TOP,
     ID_CURSOR_SHIFT_BOTTOM,
     ID_CURSOR_SHIFT_HOME,
@@ -474,11 +505,15 @@ public:
     ID_TOGGLE_OVERSTRIKE,
     ID_CURSOR_ROW,
     ID_CURSOR_COLUMN,
-    ID_CLEAN_INDENT,
+    ID_JOIN_LINES,
     ID_SHIFT_LEFT,
     ID_SHIFT_RIGHT,
     ID_SHIFT_TABLEFT,
     ID_SHIFT_TABRIGHT,
+    ID_CLEAN_INDENT,
+    ID_COPY_LINE,
+    ID_MOVE_LINE_UP,
+    ID_MOVE_LINE_DOWN,
     ID_UPPER_CASE,
     ID_LOWER_CASE,
     ID_GOTO_MATCHING,
@@ -506,23 +541,11 @@ public:
   /// Detach server-side resources
   virtual void detach();
 
-  /// Perform layout
-  virtual void layout();
+  /// Get default width
+  virtual FXint getContentWidth();
 
-  /// Return default width
-  virtual FXint getDefaultWidth();
-
-  /// Return default height
-  virtual FXint getDefaultHeight();
-
-  /// Enable the text widget
-  virtual void enable();
-
-  /// Disable the text widget
-  virtual void disable();
-
-  /// Need to recalculate size
-  virtual void recalc();
+  /// Get default height
+  virtual FXint getContentHeight();
 
   /// Return visible scroll-area x position
   virtual FXint getVisibleX() const;
@@ -536,11 +559,23 @@ public:
   /// Return visible scroll-area height
   virtual FXint getVisibleHeight() const;
 
-  /// Get default width
-  virtual FXint getContentWidth();
+  /// Return default width
+  virtual FXint getDefaultWidth();
 
-  /// Get default height
-  virtual FXint getContentHeight();
+  /// Return default height
+  virtual FXint getDefaultHeight();
+
+  /// Perform layout
+  virtual void layout();
+
+  /// Enable the text widget
+  virtual void enable();
+
+  /// Disable the text widget
+  virtual void disable();
+
+  /// Need to recalculate size
+  virtual void recalc();
 
   /// Returns true because a text widget can receive focus
   virtual FXbool canFocus() const;
@@ -713,16 +748,18 @@ public:
   /// Return number of rows in buffer
   FXint getNumRows() const { return nrows; }
 
-  /// Return entire text
+  /// Return entire text as string
   FXString getText() const;
 
-  /// Get selected text
-  FXString getSelectedText() const;
-
   /// Retrieve text into buffer
   void getText(FXchar* text,FXint n) const;
+
+  /// Retrieve text into string
   void getText(FXString& text) const;
 
+  /// Get selected text
+  FXString getSelectedText() const;
+
   /// Change the text in the buffer to new text
   virtual void setText(const FXchar* text,FXint n,FXbool notify=false);
   virtual void setText(const FXString& text,FXbool notify=false);
@@ -768,16 +805,23 @@ public:
   virtual void changeStyle(FXint pos,const FXchar* style,FXint n);
   virtual void changeStyle(FXint pos,const FXString& style);
 
-  /// Extract n bytes of text from position pos
+  /// Extract n bytes of text from position pos into already allocated buffer
   void extractText(FXchar *text,FXint pos,FXint n) const;
+
+  /// Extract n bytes of text from position pos into string text
   void extractText(FXString& text,FXint pos,FXint n) const;
 
-  /// Extract n bytes of style info from position pos
-  void extractStyle(FXchar *style,FXint pos,FXint n) const;
-  void extractStyle(FXString& text,FXint pos,FXint n) const;
+  /// Return n bytes of contents of text buffer from position pos
+  FXString extractText(FXint pos,FXint n) const;
 
-  /// Shift block of lines from position start up to end by given amount
-  FXint shiftText(FXint start,FXint end,FXint amount,FXbool notify=false);
+  /// Extract n bytes of style info from position pos into already allocated buffer
+  void extractStyle(FXchar *style,FXint pos,FXint n) const;
+
+  /// Extract n bytes of style info from position pos into string text
+  void extractStyle(FXString& style,FXint pos,FXint n) const;
+
+  /// Return n bytes of style info from buffer from position pos
+  FXString extractStyle(FXint pos,FXint n) const;
 
   /**
   * Search for string in text buffer, returning the extent of the string in beg and end.
@@ -793,24 +837,47 @@ public:
   */
   FXbool findText(const FXString& string,FXint* beg=NULL,FXint* end=NULL,FXint start=0,FXuint flags=SEARCH_FORWARD|SEARCH_WRAP|SEARCH_EXACT,FXint npar=1);
 
-  /// Return text position containing x, y coordinate
-  FXint getPosContaining(FXint x,FXint y) const;
 
   /// Return text position at given visible x,y coordinate
   FXint getPosAt(FXint x,FXint y) const;
 
-  /// Return y coordinate of pos
-  FXint getYOfPos(FXint pos) const;
+  /// Return text position containing x, y coordinate
+  FXint getPosContaining(FXint x,FXint y) const;
 
-  /// Return x coordinate of pos
+  /// Return screen x-coordinate of pos
   FXint getXOfPos(FXint pos) const;
 
-  /// Count number of rows; start should be on a row start
-  FXint countRows(FXint start,FXint end) const;
+  /// Return screen y-coordinate of pos
+  FXint getYOfPos(FXint pos) const;
 
-  /// Count number of columns; start should be on a row start
+  /**
+  * Return closest position and (row,col) of given x,y coordinate.
+  * The (row,col) is unconstrained, i.e. calculated as if tabs and
+  * area past the newline is comprised of spaces; the returned position
+  * however is inside the text.
+  * Note that when using proportional fonts, the width of a logical space
+  * inside a tab is variable, to account for the logical columns in a tab.
+  */
+  FXint getRowColumnAt(FXint x,FXint y,FXint& row,FXint& col) const;
+
+  /// Return screen x-coordinate of unconstrained (row,col).
+  FXint getXOfRowColumn(FXint row,FXint col) const;
+
+  /// Return screen y-coordinate of unconstrained (row,col).
+  FXint getYOfRowColumn(FXint row,FXint col) const;
+
+  /**
+  * Count number of columns taken up by some text.
+  * Start should be on a row start.
+  */
   FXint countCols(FXint start,FXint end) const;
 
+  /**
+  * Count number of rows taken up by some text.
+  * Start should be on a row start.
+  */
+  FXint countRows(FXint start,FXint end) const;
+
   /// Count number of newlines
   FXint countLines(FXint start,FXint end) const;
 
@@ -881,7 +948,13 @@ public:
   virtual FXbool setSelection(FXint pos,FXint len,FXbool notify=false);
 
   /// Extend the primary selection from the anchor to the given position
-  virtual FXbool extendSelection(FXint pos,FXuint select=SelectChars,FXbool notify=false);
+  virtual FXbool extendSelection(FXint pos,FXuint sel=SelectChars,FXbool notify=false);
+
+  /// Select block of characters within given box
+  virtual FXbool setBlockSelection(FXint trow,FXint lcol,FXint brow,FXint rcol,FXbool notify=false);
+
+  /// Extend primary selection from anchor to given row, column
+  virtual FXbool extendBlockSelection(FXint row,FXint col,FXbool notify=false);
 
   /// Kill or deselect primary selection
   virtual FXbool killSelection(FXbool notify=false);
@@ -901,16 +974,22 @@ public:
   /// Paste clipboard
   FXbool pasteClipboard(FXbool notify=false);
 
-  /// Replace primary selection by other text
-  FXbool replaceSelection(const FXchar *text,FXint n,FXbool notify=false);
-
   /// Replace primary selection by other text
   FXbool replaceSelection(const FXString& text,FXbool notify=false);
 
+  /// Enter text into editor as if typed
+  void enterText(const FXchar *text,FXint n,FXbool notify=false);
+
+  /// Enter text into editor as if typed
+  void enterText(const FXString& text,FXbool notify=false);
+
   /// Return true if position pos is selected
   FXbool isPosSelected(FXint pos) const;
 
-  /// Return true if position is fully visible
+  /// Return true if position pos (and column col) is selected
+  FXbool isPosSelected(FXint pos,FXint col) const;
+
+  /// Return true if line containing position is fully visible
   FXbool isPosVisible(FXint pos) const;
 
   /// Scroll text to make the given position visible
@@ -925,6 +1004,12 @@ public:
   /// Set the cursor position
   virtual void setCursorPos(FXint pos,FXbool notify=false);
 
+  /// Return the cursor position
+  FXint getCursorPos() const { return cursorpos; }
+
+  /// Set cursor row, column
+  void setCursorRowColumn(FXint row,FXint col,FXbool notify=false);
+
   /// Set cursor row
   void setCursorRow(FXint row,FXbool notify=false);
 
@@ -937,26 +1022,44 @@ public:
   /// Return cursor row, i.e. indent position
   FXint getCursorColumn() const { return cursorcol; }
 
-  /// Return the cursor position
-  FXint getCursorPos() const { return cursorpos; }
-
-  /// Move cursor to position, and scroll into view
-  void moveCursor(FXint pos,FXbool notify=false);
-
-  /// Move cursor to position, and extent the selection to this point
-  void moveCursorAndSelect(FXint pos,FXuint select,FXbool notify=false);
-
   /// Set the anchor position
   void setAnchorPos(FXint pos);
 
   /// Return the anchor position
   FXint getAnchorPos() const { return anchorpos; }
 
+  /// Set anchor row and column
+  void setAnchorRowColumn(FXint row,FXint col);
+
+  /// Return anchor row
+  FXint getAnchorRow() const { return anchorrow; }
+
+  /// Return anchor row
+  FXint getAnchorColumn() const { return anchorcol; }
+
+  /// Move cursor to position, and scroll into view
+  void moveCursor(FXint pos,FXbool notify=false);
+
+  /// Move cursor to row and column, and scroll into view
+  void moveCursorRowColumn(FXint row,FXint col,FXbool notify=false);
+
+  /// Move cursor to position, and extend the selection to this point
+  void moveCursorAndSelect(FXint pos,FXuint sel,FXbool notify=false);
+
+  /// Move cursor to row and column, and extend the block selection to this point
+  void moveCursorRowColumnAndSelect(FXint row,FXint col,FXbool notify=false);
+
   /// Return selection start position
-  FXint getSelStartPos() const { return selstartpos; }
+  FXint getSelStartPos() const { return select.startpos; }
 
   /// Return selection end position
-  FXint getSelEndPos() const { return selendpos; }
+  FXint getSelEndPos() const { return select.endpos; }
+
+  /// Return selection start column
+  FXint getSelStartColumn() const { return select.startcol; }
+
+  /// Return selection end column
+  FXint getSelEndColumn() const { return select.endcol; }
 
   /// Change text widget style
   void setTextStyle(FXuint style);
@@ -986,7 +1089,11 @@ public:
 
   /**
   * Change brace and parenthesis match highlighting time, in nanoseconds.
-  * A match highlight time of 0 disables brace matching.
+  * If highlighting for a small interval, only flash if the matching brace
+  * is visible. If the highlighting interval is set to forever, highlighting
+  * stays on till cursor moves, and the brace is highlighted even if not
+  * (yet) visible.  Note that this may be expensive as a large part of the
+  * text buffer could be visited to find the matching brace.
   */
   void setHiliteMatchTime(FXTime t){ matchtime=t; }
 
@@ -1019,7 +1126,6 @@ public:
   virtual ~FXText();
   };
 
-
 }
 
 #endif
