@@ -3,7 +3,7 @@
 *                     FOX Definitions, Types, and Macros                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2017 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2018 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU Lesser General Public License as published by   *
@@ -73,9 +73,11 @@
 #define FXIMPORT __declspec(dllimport)
 #else
 #if defined(__GNUC__) && (__GNUC__ >= 4)
+#define FXLOCAL  __attribute__((visibility("hidden")))
 #define FXEXPORT __attribute__((visibility("default")))
-#define FXIMPORT
+#define FXIMPORT __attribute__((visibility("default")))
 #else
+#define FXLOCAL
 #define FXEXPORT
 #define FXIMPORT
 #endif
